Package: gh
Title: 'GitHub' 'API'
Version: 1.6.1
Authors@R: c(
    person("Gábor", "Csárdi", , "csardi.gabor@gmail.com", role = c("cre", "ctb")),
    person("Jennifer", "Bryan", role = "aut"),
    person("Hadley", "Wickham", role = "aut"),
    person("Posit Software, PBC", role = c("cph", "fnd"),
           comment = c(ROR = "03wc8by49"))
  )
Description: Minimal client to access the 'GitHub' 'API'.
License: MIT + file LICENSE
URL: https://gh.r-lib.org/, https://github.com/r-lib/gh
BugReports: https://github.com/r-lib/gh/issues
Depends: R (>= 4.1)
Imports: cli (>= 3.0.1), gitcreds, glue, httr2 (>= 1.0.6), ini,
        jsonlite, lifecycle, rlang (>= 1.1.0)
Suggests: connectcreds, covr, knitr, rmarkdown, rprojroot, spelling,
        testthat (>= 3.0.0), vctrs, webfakes (>= 1.5.0), withr
VignetteBuilder: knitr
Config/Needs/website: tidyverse/tidytemplate
Config/testthat/edition: 3
Config/usethis/last-upkeep: 2025-04-29
Encoding: UTF-8
Language: en-US
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-07-20 09:37:27 UTC; gaborcsardi
Author: Gábor Csárdi [cre, ctb],
  Jennifer Bryan [aut],
  Hadley Wickham [aut],
  Posit Software, PBC [cph, fnd] (ROR: <https://ror.org/03wc8by49>)
Maintainer: Gábor Csárdi <csardi.gabor@gmail.com>
Repository: CRAN
Date/Publication: 2026-07-20 10:20:02 UTC
