# Generated by roxygen2: do not edit by hand

S3method(format,gh_pat)
S3method(print,gh_pat)
S3method(print,gh_response)
S3method(str,gh_pat)
S3method(vctrs::vec_cast,list.gh_response)
S3method(vctrs::vec_ptype2,gh_response.gh_response)
export(fake_github_app)
export(gh)
export(gh_first)
export(gh_gql)
export(gh_last)
export(gh_next)
export(gh_prev)
export(gh_rate_limit)
export(gh_rate_limits)
export(gh_token)
export(gh_token_exists)
export(gh_tree_remote)
export(gh_whoami)
import(rlang)
importFrom(cli,cli_status)
importFrom(cli,cli_status_update)
importFrom(glue,glue)
importFrom(jsonlite,fromJSON)
importFrom(jsonlite,prettify)
importFrom(jsonlite,toJSON)
importFrom(lifecycle,deprecated)
importFrom(utils,URLencode)
importFrom(utils,capture.output)
