#' @keywords internal
#' @aliases gh-package
"_PACKAGE"

# The following block is used by usethis to automatically manage
# roxygen namespace tags. Modify with care!
## usethis namespace: start
#' @import rlang
#' @importFrom cli cli_status cli_status_update
#' @importFrom glue glue
#' @importFrom jsonlite fromJSON toJSON
#' @importFrom lifecycle deprecated
#' @importFrom utils URLencode capture.output
## usethis namespace: end
NULL
