---
name: "\U0001F41B Bug report"
about: Report a bug or unexpected behavior while using GitHub CLI
title: ''
labels: bug
assignees: ''

---

### Describe the bug

A clear and concise description of what the bug is. Include version by typing `gh --version`.

### Steps to reproduce the behavior

1. Type this '...'
2. View the output '....'
3. See error

### Expected vs actual behavior

A clear and concise description of what you expected to happen and what actually happened.

### Logs

Paste the activity from your command line. Redact if needed.
