# errors

    Code
      gh_rate_limit(list())
    Condition
      Error in `gh_rate_limit()`:
      ! `response` must be a <gh_response> object, not an empty list.
    Code
      gh_rate_limits(.token = "bad")
    Condition
      Error in `gh()`:
      ! GitHub API error (401): Bad credentials
      i Read more at <https://docs.github.com/rest>

