# can print all types of object

    Code
      json
    Output
      {
        "name": "LICENSE",
        "path": "LICENSE",
        "type": "file",
        "content": "ZmFrZQ==",
        "encoding": "base64"
      } 
    Code
      file
    Output
      [1] "LICENSE"
      attr(,"class")
      [1] "gh_response" "path"       
    Code
      raw
    Output
       [1] 50 61 63 6b 61 67 65 3a 20 67 68 0a 54 69 74 6c 65 3a 20 46 61 6b 65 20 63
      [26] 6f 6e 74 65 6e 74 20 6f 66 20 4c 49 43 45 4e 53 45 0a
      attr(,"class")
      [1] "gh_response" "raw"        

